@@ -1,13 +1,10 @@
 from bibtutils.version import __version__
 __all__ = [
-    'sanitycheck',
     'gcp',
     'slack'
-
 ]
 
 from bibtutils import gcp
 from bibtutils import slack
-from bibtutils import sanitycheck
 
 '''A Python package featuring functionality often used by BITS Blue Team (BIBT).'''
\ No newline at end of file
