@@ -1,8 +1,10 @@
 from bibtutils.version import __version__
 __all__ = [
-    'alert'
+    'error',
+    'message'
 ]
 
-from bibtutils.slack import alert
+from bibtutils.slack import error
+from bibtutils.slack import message
 
 '''Slack-related functionality for BIBT.'''
\ No newline at end of file
